using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;
using Syncfusion.Grouping;

using ISummary = Syncfusion.Collections.BinaryTree.ITreeTableSummary;
using System.Collections;

namespace WindowsApplication124
{
    public partial class Form1 : Form
    {
        public Form1()
        {
             InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

            #region Get the DataSource
            DataTable dt = new DataTable("MyTable");

            int nCols = 4;
            int nRows = 20;
            Random r = new Random(123345345);

            for (int i = 0; i < nCols; i++)
                dt.Columns.Add(new DataColumn(string.Format("Col{0}", i), typeof(int)));

            for (int i = 0; i < nRows; ++i)
            {
                DataRow dr = dt.NewRow();
                for (int j = 0; j < nCols; j++)
                    dr[j] = r.Next(10) + 1;
                dt.Rows.Add(dr);
            }
            #endregion

            this.gridGroupingControl1.DataSource = dt;

            //needed to setup the CustomSummary type
            this.gridGroupingControl1.TableDescriptor.QueryCustomSummary += new GridQueryCustomSummaryEventHandler(gridGroupingControl1_QueryCustomSummary);

            //use a code naming convention to pass the column holding the weights into the SummaryDescriptor

            //weighted sum of Col2 using weights from Col3
            GridSummaryColumnDescriptor wgtSumCol2 = GetWeightedSummaryColumnDescriptor("Col2", "Col3");
            


           
            //add the summary columns to a summary row
            GridSummaryRowDescriptor srdRow0 = new GridSummaryRowDescriptor("Row0", "Weighted Summary", new GridSummaryColumnDescriptor[] { wgtSumCol2 });
            srdRow0.Visible = false;
            srdRow0.Appearance.GroupCaptionSummaryCell.BackColor = Color.LightGoldenrodYellow;
            srdRow0.Appearance.GroupCaptionSummaryCell.HorizontalAlignment = GridHorizontalAlignment.Right;
            this.gridGroupingControl1.TableDescriptor.SummaryRows.Add(srdRow0);

            this.gridGroupingControl1.TableDescriptor.Columns["Col0"].Width = 130;

            //put the summary row in the caption
            this.gridGroupingControl1.TableDescriptor.ChildGroupOptions.ShowCaptionSummaryCells = true;
            this.gridGroupingControl1.TableDescriptor.ChildGroupOptions.CaptionSummaryRow = "Row0";

            //group by Column 1 but order the groups by the weighted sum
            SortColumnDescriptor gsd = new SortColumnDescriptor("Col1");

            // specify a summary name and the property (values will be determined using reflection)
            //gsd.SetGroupSummarySortOrder(wgtSumCol2.GetSummaryDescriptorName(), "WeightedAverage");

            //or specify a IComparer object
            gsd.GroupSortOrderComparer = new MyComparer(wgtSumCol2.SummaryDescriptor, "WeightedAverage");

            this.gridGroupingControl1.TableDescriptor.GroupedColumns.Add(gsd);

             this.gridGroupingControl1.Table.ExpandAllGroups();

          

        }

      
        //custom summary
        private void gridGroupingControl1_QueryCustomSummary(object sender, GridQueryCustomSummaryEventArgs e)
        {
            if (e.SummaryDescriptor.SummaryType == SummaryType.Custom)
            {
                e.SummaryDescriptor.CreateSummaryMethod =
                    new CreateSummaryDelegate(WeightedSummary.CreateSummaryMethod);
            }
            e.Handled = true;
        }

        private GridSummaryColumnDescriptor GetWeightedSummaryColumnDescriptor(string sourceCol, string weightCol)
        {
            GridSummaryColumnDescriptor wgtSumCol = new GridSummaryColumnDescriptor();
            wgtSumCol.Name = string.Format("{0}_{1}", sourceCol, weightCol); //special name following the convention above
            wgtSumCol.DataMember = sourceCol; //the column this summary is applied to
            wgtSumCol.DisplayColumn = sourceCol; //where thissummary is displayed
            wgtSumCol.Format = "{WeightedAverage:#.##}"; //what is displayed in the summary
            wgtSumCol.SummaryType = SummaryType.Custom; //marks this as a CustomSummary
            wgtSumCol.Appearance.AnySummaryCell.HorizontalAlignment = GridHorizontalAlignment.Right;
            wgtSumCol.MaxLength = 6;

            return wgtSumCol;
        }

      
    }

    public class MyComparer : IComparer
    {
        private SummaryDescriptor sd;
        private string name;

        public MyComparer(SummaryDescriptor sd, string name)
        {
            this.sd = sd;
            this.name = name;
        }

        #region IComparer Members

        public int Compare(object x, object y)
        {
            Group xg = x as Group;
            Group yg = y as Group;
            if (xg == null && yg == null)
                return 0;
            else if (xg == null)
                return -1;
            else if (yg == null)
                return 1;

            IComparable yc = yg.GetSummaryProperty(sd, name) as IComparable;
            IComparable xc = xg.GetSummaryProperty(sd, name) as IComparable;
            return xc.CompareTo(yc);
        }

        #endregion
    }
}